@@ -31,6 +31,7 @@
 #include "filter.h"
 #include "keydb.h"
 #include "../common/status.h"
+#include "exec.h"
 #include "main.h"
 #include "../common/i18n.h"
 #include "../common/ttyio.h"
@@ -99,13 +100,15 @@ static struct parse_options keyserver_opts[]=
      N_("automatically retrieve keys when verifying signatures")},
     {"honor-keyserver-url",KEYSERVER_HONOR_KEYSERVER_URL,NULL,
      N_("honor the preferred keyserver URL set on the key")},
+    {"honor-pka-record",KEYSERVER_HONOR_PKA_RECORD,NULL,
+     N_("honor the PKA record set on a key when retrieving keys")},
     {NULL,0,NULL,NULL}
   };
 
 static gpg_error_t keyserver_get (ctrl_t ctrl,
                                   KEYDB_SEARCH_DESC *desc, int ndesc,
                                   struct keyserver_spec *override_keyserver,
-                                  unsigned int flags,
+                                  int quick,
                                   unsigned char **r_fpr, size_t *r_fprlen);
 static gpg_error_t keyserver_put (ctrl_t ctrl, strlist_t keyspecs);
 
@@ -183,16 +186,36 @@ void
 free_keyserver_spec(struct keyserver_spec *keyserver)
 {
   xfree(keyserver->uri);
+  xfree(keyserver->scheme);
+  xfree(keyserver->auth);
+  xfree(keyserver->host);
+  xfree(keyserver->port);
+  xfree(keyserver->path);
+  xfree(keyserver->opaque);
+  free_strlist(keyserver->options);
   xfree(keyserver);
 }
 
 /* Return 0 for match */
 static int
-cmp_keyserver_spec(struct keyserver_spec *one, struct keyserver_spec *two)
+cmp_keyserver_spec(struct keyserver_spec *one,struct keyserver_spec *two)
 {
-  return !!ascii_strcasecmp(one->uri, two->uri);
-}
+  if(ascii_strcasecmp(one->scheme,two->scheme)==0)
+    {
+      if(one->host && two->host && ascii_strcasecmp(one->host,two->host)==0)
+	{
+	  if((one->port && two->port
+	      && ascii_strcasecmp(one->port,two->port)==0)
+	     || (!one->port && !two->port))
+	    return 0;
+	}
+      else if(one->opaque && two->opaque
+	      && ascii_strcasecmp(one->opaque,two->opaque)==0)
+	return 0;
+    }
 
+  return 1;
+}
 
 /* Try and match one of our keyservers.  If we can, return that.  If
    we can't, return our input. */
@@ -208,24 +231,40 @@ keyserver_match(struct keyserver_spec *spec)
   return spec;
 }
 
+/* TODO: once we cut over to an all-curl world, we don't need this
+   parser any longer so it can be removed, or at least moved to
+   keyserver/ksutil.c for limited use in gpgkeys_ldap or the like. */
 
-/* Create a new keyserver object from STRING.  Unless REQUIRE_SCHEME
- * is set a missing scheme is replaced by "hkp://".  The data structure
- * could be much easier but in the past we parsed the URI here for the
- * old 2.0 keyserver helpers - which is not anymore needed.  */
 keyserver_spec_t
-parse_keyserver_uri (const char *string, int require_scheme)
+parse_keyserver_uri (const char *string,int require_scheme)
 {
+  int assume_hkp=0;
   struct keyserver_spec *keyserver;
   const char *idx;
   int count;
+  char *uri, *duped_uri, *options;
 
   log_assert (string);
 
-  keyserver = xcalloc (1, sizeof *keyserver);
+  keyserver=xmalloc_clear(sizeof(struct keyserver_spec));
+
+  duped_uri = uri = xstrdup (string);
+
+  options=strchr(uri,' ');
+  if(options)
+    {
+      char *tok;
+
+      *options='\0';
+      options++;
+
+      while((tok=optsep(&options)))
+	warn_kshelper_option (tok, 0);
+    }
 
   /* Get the scheme */
-  for(idx=string, count=0; *idx && *idx!=':';idx++)
+
+  for(idx=uri,count=0;*idx && *idx!=':';idx++)
     {
       count++;
 
@@ -248,24 +287,165 @@ parse_keyserver_uri (const char *string, int require_scheme)
   if(*idx=='\0' || *idx=='[')
     {
       if(require_scheme)
-	goto fail;
+	return NULL;
 
       /* Assume HKP if there is no scheme */
-      keyserver->uri = xstrconcat ("hkp://", string, NULL);
+      assume_hkp=1;
+      keyserver->scheme=xstrdup("hkp");
+
+      keyserver->uri=xmalloc(strlen(keyserver->scheme)+3+strlen(uri)+1);
+      strcpy(keyserver->uri,keyserver->scheme);
+      strcat(keyserver->uri,"://");
+      strcat(keyserver->uri,uri);
     }
   else
     {
-      keyserver->uri = xstrdup (string);
+      int i;
+
+      keyserver->uri=xstrdup(uri);
+
+      keyserver->scheme=xmalloc(count+1);
+
+      /* Force to lowercase */
+      for(i=0;i<count;i++)
+	keyserver->scheme[i]=ascii_tolower(uri[i]);
+
+      keyserver->scheme[i]='\0';
+
+      /* Skip past the scheme and colon */
+      uri+=count+1;
     }
 
+  if(ascii_strcasecmp(keyserver->scheme,"x-broken-hkp")==0)
+    {
+      log_info ("keyserver option '%s' is obsolete\n",
+                "x-broken-hkp");
+    }
+  else if(ascii_strcasecmp(keyserver->scheme,"x-hkp")==0)
+    {
+      /* Canonicalize this to "hkp" so it works with both the internal
+	 and external keyserver interface. */
+      xfree(keyserver->scheme);
+      keyserver->scheme=xstrdup("hkp");
+    }
+
+  if (uri[0]=='/' && uri[1]=='/' && uri[2] == '/')
+    {
+      /* Three slashes means network path with a default host name.
+         This is a hack because it does not crok all possible
+         combiantions.  We should better repalce all code bythe parser
+         from http.c.  */
+      keyserver->path = xstrdup (uri+2);
+    }
+  else if(assume_hkp || (uri[0]=='/' && uri[1]=='/'))
+    {
+      /* Two slashes means network path. */
+
+      /* Skip over the "//", if any */
+      if(!assume_hkp)
+	uri+=2;
+
+      /* Do we have userinfo auth data present? */
+      for(idx=uri,count=0;*idx && *idx!='@' && *idx!='/';idx++)
+	count++;
+
+      /* We found a @ before the slash, so that means everything
+	 before the @ is auth data. */
+      if(*idx=='@')
+	{
+	  if(count==0)
+	    goto fail;
+
+	  keyserver->auth=xmalloc(count+1);
+	  strncpy(keyserver->auth,uri,count);
+	  keyserver->auth[count]='\0';
+	  uri+=count+1;
+	}
+
+      /* Is it an RFC-2732 ipv6 [literal address] ? */
+      if(*uri=='[')
+	{
+	  for(idx=uri+1,count=1;*idx
+		&& ((isascii (*idx) && isxdigit(*idx))
+                    || *idx==':' || *idx=='.');idx++)
+	    count++;
+
+	  /* Is the ipv6 literal address terminated? */
+	  if(*idx==']')
+	    count++;
+	  else
+	    goto fail;
+	}
+      else
+	for(idx=uri,count=0;*idx && *idx!=':' && *idx!='/';idx++)
+	  count++;
+
+      if(count==0)
+	goto fail;
+
+      keyserver->host=xmalloc(count+1);
+      strncpy(keyserver->host,uri,count);
+      keyserver->host[count]='\0';
+
+      /* Skip past the host */
+      uri+=count;
+
+      if(*uri==':')
+	{
+	  /* It would seem to be reasonable to limit the range of the
+	     ports to values between 1-65535, but RFC 1738 and 1808
+	     imply there is no limit.  Of course, the real world has
+	     limits. */
+
+	  for(idx=uri+1,count=0;*idx && *idx!='/';idx++)
+	    {
+	      count++;
+
+	      /* Ports are digits only */
+	      if(!digitp(idx))
+		goto fail;
+	    }
+
+	  keyserver->port=xmalloc(count+1);
+	  strncpy(keyserver->port,uri+1,count);
+	  keyserver->port[count]='\0';
+
+	  /* Skip past the colon and port number */
+	  uri+=1+count;
+	}
+
+      /* Everything else is the path */
+      if(*uri)
+	keyserver->path=xstrdup(uri);
+      else
+	keyserver->path=xstrdup("/");
+
+      if(keyserver->path[1])
+	keyserver->flags.direct_uri=1;
+    }
+  else if(uri[0]!='/')
+    {
+      /* No slash means opaque.  Just record the opaque blob and get
+	 out. */
+      keyserver->opaque=xstrdup(uri);
+    }
+  else
+    {
+      /* One slash means absolute path.  We don't need to support that
+	 yet. */
+      goto fail;
+    }
+
+  xfree (duped_uri);
   return keyserver;
 
  fail:
   free_keyserver_spec(keyserver);
+
+  xfree (duped_uri);
   return NULL;
 }
 
-
 struct keyserver_spec *
 parse_preferred_keyserver(PKT_signature *sig)
 {
@@ -273,7 +453,7 @@ parse_preferred_keyserver(PKT_signature *sig)
   const byte *p;
   size_t plen;
 
-  p = parse_sig_subpkt (sig, 1, SIGSUBPKT_PREF_KS, &plen);
+  p=parse_sig_subpkt(sig->hashed,SIGSUBPKT_PREF_KS,&plen);
   if(p && plen)
     {
       byte *dupe=xmalloc(plen+1);
@@ -290,6 +470,7 @@ parse_preferred_keyserver(PKT_signature *sig)
 static void
 print_keyrec (ctrl_t ctrl, int number,struct keyrec *keyrec)
 {
+  int i;
 
   iobuf_writebyte(keyrec->uidbuf,0);
   iobuf_flush_temp(keyrec->uidbuf);
@@ -328,11 +509,20 @@ print_keyrec (ctrl_t ctrl, int number,struct keyrec *keyrec)
       es_printf ("key %s",keystr(keyrec->desc.u.kid));
       break;
 
-    case KEYDB_SEARCH_MODE_FPR:
+      /* If it gave us a PGP 2.x fingerprint, not much we can do
+	 beyond displaying it. */
+    case KEYDB_SEARCH_MODE_FPR16:
+      es_printf ("key ");
+      for(i=0;i<16;i++)
+	es_printf ("%02X",keyrec->desc.u.fpr[i]);
+      break;
+
+      /* If we get a modern fingerprint, we have the most
+	 flexibility. */
+    case KEYDB_SEARCH_MODE_FPR20:
       {
 	u32 kid[2];
-	keyid_from_fingerprint (ctrl, keyrec->desc.u.fpr, keyrec->desc.fprlen,
-                                kid);
+	keyid_from_fingerprint (ctrl, keyrec->desc.u.fpr,20,kid);
 	es_printf("key %s",keystr(kid));
       }
       break;
@@ -423,7 +613,8 @@ parse_keyrec(char *keystring)
       err = classify_user_id (tok, &work->desc, 1);
       if (err || (work->desc.mode    != KEYDB_SEARCH_MODE_SHORT_KID
                   && work->desc.mode != KEYDB_SEARCH_MODE_LONG_KID
-                  && work->desc.mode != KEYDB_SEARCH_MODE_FPR))
+                  && work->desc.mode != KEYDB_SEARCH_MODE_FPR16
+                  && work->desc.mode != KEYDB_SEARCH_MODE_FPR20))
 	{
 	  work->desc.mode=KEYDB_SEARCH_MODE_NONE;
 	  return ret;
@@ -804,7 +995,8 @@ keyserver_export (ctrl_t ctrl, strlist_t users)
       err = classify_user_id (users->d, &desc, 1);
       if (err || (desc.mode    != KEYDB_SEARCH_MODE_SHORT_KID
                   && desc.mode != KEYDB_SEARCH_MODE_LONG_KID
-                  && desc.mode != KEYDB_SEARCH_MODE_FPR))
+                  && desc.mode != KEYDB_SEARCH_MODE_FPR16
+                  && desc.mode != KEYDB_SEARCH_MODE_FPR20))
 	{
 	  log_error(_("\"%s\" not a key ID: skipping\n"),users->d);
 	  continue;
@@ -873,10 +1065,14 @@ keyserver_retrieval_screener (kbnode_t keyblock, void *opaque)
       /* Compare requested and returned fingerprints if available. */
       for (n = 0; n < ndesc; n++)
         {
-          if (desc[n].mode == KEYDB_SEARCH_MODE_FPR)
+          if (desc[n].mode == KEYDB_SEARCH_MODE_FPR20)
             {
-              if (fpr_len == desc[n].fprlen
-                  && !memcmp (fpr, desc[n].u.fpr, desc[n].fprlen))
+              if (fpr_len == 20 && !memcmp (fpr, desc[n].u.fpr, 20))
+                return 0;
+            }
+          else if (desc[n].mode == KEYDB_SEARCH_MODE_FPR16)
+            {
+              if (fpr_len == 16 && !memcmp (fpr, desc[n].u.fpr, 16))
                 return 0;
             }
           else if (desc[n].mode == KEYDB_SEARCH_MODE_LONG_KID)
@@ -914,7 +1110,8 @@ keyserver_import (ctrl_t ctrl, strlist_t users)
       err = classify_user_id (users->d, &desc[count], 1);
       if (err || (desc[count].mode    != KEYDB_SEARCH_MODE_SHORT_KID
                   && desc[count].mode != KEYDB_SEARCH_MODE_LONG_KID
-                  && desc[count].mode != KEYDB_SEARCH_MODE_FPR))
+                  && desc[count].mode != KEYDB_SEARCH_MODE_FPR16
+                  && desc[count].mode != KEYDB_SEARCH_MODE_FPR20))
 	{
 	  log_error (_("\"%s\" not a key ID: skipping\n"), users->d);
 	  continue;
@@ -945,72 +1142,48 @@ keyserver_any_configured (ctrl_t ctrl)
 }
 
 
-/* Import all keys that exactly match MBOX */
-gpg_error_t
-keyserver_import_mbox (ctrl_t ctrl, const char *mbox,
+/* Import all keys that exactly match NAME */
+int
+keyserver_import_name (ctrl_t ctrl, const char *name,
                        unsigned char **fpr, size_t *fprlen,
-                       struct keyserver_spec *keyserver, unsigned int flags)
-{
-  KEYDB_SEARCH_DESC desc = { 0 };
-
-  desc.mode = KEYDB_SEARCH_MODE_MAIL;
-  desc.u.name = mbox;
-
-  return keyserver_get (ctrl, &desc, 1, keyserver, flags, fpr, fprlen);
-}
-
-
-/* Import the keys that match exactly MBOX */
-int
-keyserver_import_ntds (ctrl_t ctrl, const char *mbox,
-                       unsigned char **fpr, size_t *fprlen)
-{
-  KEYDB_SEARCH_DESC desc = { 0 };
-  struct keyserver_spec keyserver = { NULL, "ldap:///" };
-
-  desc.mode = KEYDB_SEARCH_MODE_MAIL;
-  desc.u.name = mbox;
-
-  return keyserver_get (ctrl, &desc, 1, &keyserver, 0, fpr, fprlen);
-}
-
-
-int
-keyserver_import_fprint (ctrl_t ctrl, const byte *fprint, size_t fprint_len,
-			 struct keyserver_spec *keyserver,
-                         unsigned int flags)
+                       struct keyserver_spec *keyserver)
 {
   KEYDB_SEARCH_DESC desc;
 
-  memset (&desc, 0, sizeof(desc));
+  memset (&desc, 0, sizeof desc);
 
-  if (fprint_len == 16 || fprint_len == 20 || fprint_len == 32)
-    desc.mode = KEYDB_SEARCH_MODE_FPR;
-  else
-    return gpg_error (GPG_ERR_INV_ARG);
+  desc.mode = KEYDB_SEARCH_MODE_EXACT;
+  desc.u.name = name;
 
-  memcpy (desc.u.fpr, fprint, fprint_len);
-  desc.fprlen = fprint_len;
-
-  return keyserver_get (ctrl, &desc, 1, keyserver, flags, NULL, NULL);
+  return keyserver_get (ctrl, &desc, 1, keyserver, 0, fpr, fprlen);
 }
 
 
 int
-keyserver_import_fprint_ntds (ctrl_t ctrl,
-                              const byte *fprint, size_t fprint_len)
+keyserver_import_fprint (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
+			 struct keyserver_spec *keyserver, int quick)
 {
-  struct keyserver_spec keyserver = { NULL, "ldap:///" };
+  KEYDB_SEARCH_DESC desc;
 
-  return keyserver_import_fprint (ctrl, fprint, fprint_len,
-                                  &keyserver, KEYSERVER_IMPORT_FLAG_LDAP);
+  memset(&desc,0,sizeof(desc));
+
+  if(fprint_len==16)
+    desc.mode=KEYDB_SEARCH_MODE_FPR16;
+  else if(fprint_len==20)
+    desc.mode=KEYDB_SEARCH_MODE_FPR20;
+  else
+    return -1;
+
+  memcpy(desc.u.fpr,fprint,fprint_len);
+
+  /* TODO: Warn here if the fingerprint we got doesn't match the one
+     we asked for? */
+  return keyserver_get (ctrl, &desc, 1, keyserver, quick, NULL, NULL);
 }
 
-
 int
 keyserver_import_keyid (ctrl_t ctrl,
-                        u32 *keyid,struct keyserver_spec *keyserver,
-                        unsigned int flags)
+                        u32 *keyid,struct keyserver_spec *keyserver, int quick)
 {
   KEYDB_SEARCH_DESC desc;
 
@@ -1020,14 +1193,14 @@ keyserver_import_keyid (ctrl_t ctrl,
   desc.u.kid[0]=keyid[0];
   desc.u.kid[1]=keyid[1];
 
-  return keyserver_get (ctrl, &desc, 1, keyserver, flags, NULL, NULL);
+  return keyserver_get (ctrl, &desc, 1, keyserver, quick, NULL, NULL);
 }
 
 
 /* code mostly stolen from do_export_stream */
 static int
 keyidlist (ctrl_t ctrl, strlist_t users, KEYDB_SEARCH_DESC **klist,
-           int *count)
+           int *count, int fakev3)
 {
   int rc = 0;
   int num = 100;
@@ -1042,7 +1215,7 @@ keyidlist (ctrl_t ctrl, strlist_t users, KEYDB_SEARCH_DESC **klist,
 
   *klist=xmalloc(sizeof(KEYDB_SEARCH_DESC)*num);
 
-  kdbhd = keydb_new (ctrl);
+  kdbhd = keydb_new ();
   if (!kdbhd)
     {
       rc = gpg_error_from_syserror ();
@@ -1092,25 +1265,46 @@ keyidlist (ctrl_t ctrl, strlist_t users, KEYDB_SEARCH_DESC **klist,
 
       if((node=find_kbnode(keyblock,PKT_PUBLIC_KEY)))
 	{
+	  /* This is to work around a bug in some keyservers (pksd and
+             OKS) that calculate v4 RSA keyids as if they were v3 RSA.
+             The answer is to refresh both the correct v4 keyid
+             (e.g. 99242560) and the fake v3 keyid (e.g. 68FDDBC7).
+             This only happens for key refresh using the HKP scheme
+             and if the refresh-add-fake-v3-keyids keyserver option is
+             set. */
+	  if(fakev3 && is_RSA(node->pkt->pkt.public_key->pubkey_algo) &&
+	     node->pkt->pkt.public_key->version>=4)
+	    {
+	      (*klist)[*count].mode=KEYDB_SEARCH_MODE_LONG_KID;
+	      v3_keyid (node->pkt->pkt.public_key->pkey[0],
+                        (*klist)[*count].u.kid);
+	      (*count)++;
+
+	      if(*count==num)
+		{
+		  num+=100;
+		  *klist=xrealloc(*klist,sizeof(KEYDB_SEARCH_DESC)*num);
+		}
+	    }
+
 	  /* v4 keys get full fingerprints.  v3 keys get long keyids.
              This is because it's easy to calculate any sort of keyid
              from a v4 fingerprint, but not a v3 fingerprint. */
 
-	  if (node->pkt->pkt.public_key->version < 4)
+	  if(node->pkt->pkt.public_key->version<4)
 	    {
 	      (*klist)[*count].mode=KEYDB_SEARCH_MODE_LONG_KID;
 	      keyid_from_pk(node->pkt->pkt.public_key,
 			    (*klist)[*count].u.kid);
 	    }
 	  else
-            {
-	      size_t fprlen;
+	    {
+	      size_t dummy;
 
-	      fingerprint_from_pk (node->pkt->pkt.public_key,
-                                   (*klist)[*count].u.fpr, &fprlen);
-              (*klist)[*count].mode = KEYDB_SEARCH_MODE_FPR;
-              (*klist)[*count].fprlen = fprlen;
-            }
+	      (*klist)[*count].mode=KEYDB_SEARCH_MODE_FPR20;
+	      fingerprint_from_pk(node->pkt->pkt.public_key,
+				  (*klist)[*count].u.fpr,&dummy);
+	    }
 
 	  /* This is a little hackish, using the skipfncvalue as a
 	     void* pointer to the keyserver spec, but we don't need
@@ -1183,6 +1377,7 @@ keyserver_refresh (ctrl_t ctrl, strlist_t users)
 {
   gpg_error_t err;
   int count, numdesc;
+  int fakev3 = 0;
   KEYDB_SEARCH_DESC *desc;
   unsigned int options=opt.keyserver_options.import_options;
 
@@ -1196,8 +1391,19 @@ keyserver_refresh (ctrl_t ctrl, strlist_t users)
      the end here. */
   opt.keyserver_options.import_options|=IMPORT_FAST;
 
+  /* If refresh_add_fake_v3_keyids is on and it's a HKP or MAILTO
+     scheme, then enable fake v3 keyid generation.  Note that this
+     works only with a keyserver configured. gpg.conf
+     (i.e. opt.keyserver); however that method of configuring a
+     keyserver is deprecated and in any case it is questionable
+     whether we should keep on supporting these ancient and broken
+     keyservers.  */
+  if((opt.keyserver_options.options&KEYSERVER_ADD_FAKE_V3) && opt.keyserver
+     && (ascii_strcasecmp(opt.keyserver->scheme,"hkp")==0 ||
+	 ascii_strcasecmp(opt.keyserver->scheme,"mailto")==0))
+    fakev3=1;
 
-  err = keyidlist (ctrl, users, &desc, &numdesc);
+  err = keyidlist (ctrl, users, &desc, &numdesc, fakev3);
   if (err)
     return err;
 
@@ -1214,9 +1420,7 @@ keyserver_refresh (ctrl_t ctrl, strlist_t users)
 	      struct keyserver_spec *keyserver=desc[i].skipfncvalue;
 
               if (!opt.quiet)
-                log_info (ngettext("refreshing %d key from %s\n",
-                                   "refreshing %d keys from %s\n",
-                                   1), 1, keyserver->uri);
+                log_info (_("refreshing %d key from %s\n"), 1, keyserver->uri);
 
 	      /* We use the keyserver structure we parsed out before.
 		 Note that a preferred keyserver without a scheme://
@@ -1286,6 +1490,16 @@ keyserver_search (ctrl_t ctrl, strlist_t tokens)
   if (!tokens)
     return 0;  /* Return success if no patterns are given.  */
 
+  /* Write global options */
+
+  /* for(temp=opt.keyserver_options.other;temp;temp=temp->next) */
+  /*   es_fprintf(spawn->tochild,"OPTION %s\n",temp->d); */
+
+  /* Write per-keyserver options */
+
+  /* for(temp=keyserver->options;temp;temp=temp->next) */
+  /*   es_fprintf(spawn->tochild,"OPTION %s\n",temp->d); */
+
   {
     membuf_t mb;
     strlist_t item;
@@ -1305,6 +1519,8 @@ keyserver_search (ctrl_t ctrl, strlist_t tokens)
         goto leave;
       }
   }
+  /* FIXME: Enable the next line */
+  /* log_info (_("searching for \"%s\" from %s\n"), searchstr, keyserver->uri); */
 
   parm.ctrl = ctrl;
   if (searchstr)
@@ -1326,6 +1542,31 @@ keyserver_search (ctrl_t ctrl, strlist_t tokens)
   else if (err)
     log_error ("error searching keyserver: %s\n", gpg_strerror (err));
 
+  /* switch(ret) */
+  /*   { */
+  /*   case KEYSERVER_SCHEME_NOT_FOUND: */
+  /*     log_error(_("no handler for keyserver scheme '%s'\n"), */
+  /*   	    opt.keyserver->scheme); */
+  /*     break; */
+
+  /*   case KEYSERVER_NOT_SUPPORTED: */
+  /*     log_error(_("action '%s' not supported with keyserver " */
+  /*   	      "scheme '%s'\n"), "search", opt.keyserver->scheme); */
+  /*     break; */
+
+  /*   case KEYSERVER_TIMEOUT: */
+  /*     log_error(_("keyserver timed out\n")); */
+  /*     break; */
+
+  /*   case KEYSERVER_INTERNAL_ERROR: */
+  /*   default: */
+  /*     log_error(_("keyserver internal error\n")); */
+  /*     break; */
+  /*   } */
+
+  /* return gpg_error (GPG_ERR_KEYSERVER); */
+
+
  leave:
   xfree (parm.desc);
   xfree (parm.searchstr_disp);
@@ -1343,7 +1584,7 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
                      int *r_ndesc_used,
                      import_stats_t stats_handle,
                      struct keyserver_spec *override_keyserver,
-                     unsigned int flags,
+                     int quick,
                      unsigned char **r_fpr, size_t *r_fprlen)
 
 {
@@ -1372,14 +1613,15 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
      single request will be rejected only later by gpg_dirmngr_ks_get
      but we are sure that R_NDESC_USED has been updated.  This avoids
      a possible indefinite loop.  */
-  linelen = 24; /* "KS_GET --quick --ldap --" */
+  linelen = 17; /* "KS_GET --quick --" */
   for (npat=npat_fpr=0, idx=0; idx < ndesc; idx++)
     {
       int quiet = 0;
 
-      if (desc[idx].mode == KEYDB_SEARCH_MODE_FPR)
+      if (desc[idx].mode == KEYDB_SEARCH_MODE_FPR20
+          || desc[idx].mode == KEYDB_SEARCH_MODE_FPR16)
         {
-          n = 1+2+2*desc[idx].fprlen;
+          n = 1+2+2*20;
           if (idx && linelen + n > MAX_KS_GET_LINELEN)
             break; /* Declare end of this chunk.  */
           linelen += n;
@@ -1390,9 +1632,11 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
           else
             {
               strcpy (pattern[npat], "0x");
-              bin2hex (desc[idx].u.fpr, desc[idx].fprlen, pattern[npat]+2);
+              bin2hex (desc[idx].u.fpr,
+                       desc[idx].mode == KEYDB_SEARCH_MODE_FPR20? 20 : 16,
+                       pattern[npat]+2);
               npat++;
-              if (desc[idx].fprlen == 20 || desc[idx].fprlen == 32)
+              if (desc[idx].mode == KEYDB_SEARCH_MODE_FPR20)
                 npat_fpr++;
             }
         }
@@ -1446,25 +1690,6 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
               quiet = 1;
             }
         }
-      else if(desc[idx].mode == KEYDB_SEARCH_MODE_MAIL)
-        {
-          n = 1 + strlen (desc[idx].u.name) + 1 + 1;
-          if (idx && linelen + n > MAX_KS_GET_LINELEN)
-            break; /* Declare end of this chunk.  */
-          linelen += n;
-
-          if (desc[idx].u.name[0] == '<')
-            pattern[npat] = xtrystrdup (desc[idx].u.name);
-          else
-            pattern[npat] = strconcat ("<", desc[idx].u.name, ">", NULL);
-          if (!pattern[npat])
-            err = gpg_error_from_syserror ();
-          else
-            {
-              npat++;
-              quiet = 1;
-            }
-        }
       else if (desc[idx].mode == KEYDB_SEARCH_MODE_NONE)
         continue;
       else
@@ -1480,18 +1705,23 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
 
       if (!quiet && override_keyserver)
         {
-          log_info (_("requesting key %s from %s\n"),
-                    keystr_from_desc (&desc[idx]), override_keyserver->uri);
+          if (override_keyserver->host)
+            log_info (_("requesting key %s from %s server %s\n"),
+                      keystr_from_desc (&desc[idx]),
+                      override_keyserver->scheme, override_keyserver->host);
+          else
+            log_info (_("requesting key %s from %s\n"),
+                      keystr_from_desc (&desc[idx]), override_keyserver->uri);
         }
     }
 
-  /* Remember how many of the search items were considered.  Note that
+  /* Remember now many of search items were considered.  Note that
      this is different from NPAT.  */
   *r_ndesc_used = idx;
 
   only_fprs = (npat && npat == npat_fpr);
 
-  err = gpg_dirmngr_ks_get (ctrl, pattern, override_keyserver, flags,
+  err = gpg_dirmngr_ks_get (ctrl, pattern, override_keyserver, quick,
                             &datastream, &source);
   for (idx=0; idx < npat; idx++)
     xfree (pattern[idx]);
@@ -1499,12 +1729,9 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
   if (opt.verbose && source)
     log_info ("data source: %s\n", source);
 
-
-
   if (!err)
     {
       struct ks_retrieval_screener_arg_s screenerarg;
-      unsigned int options;
 
       /* FIXME: Check whether this comment should be moved to dirmngr.
 
@@ -1518,22 +1745,12 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
          never accept or send them but we better protect against rogue
          keyservers. */
 
-      /* For LDAP servers we reset IMPORT_SELF_SIGS_ONLY and
-       * IMPORT_CLEAN unless they have been set explicitly.  */
-      options = (opt.keyserver_options.import_options | IMPORT_ONLY_PUBKEYS);
-      if (source && (!strncmp (source, "ldap:", 5)
-                     || !strncmp (source, "ldaps:", 6)))
-        {
-          if (!opt.flags.expl_import_self_sigs_only)
-            options &= ~IMPORT_SELF_SIGS_ONLY;
-          if (!opt.flags.expl_import_clean)
-            options &= ~IMPORT_CLEAN;
-        }
-
       screenerarg.desc = desc;
       screenerarg.ndesc = *r_ndesc_used;
       import_keys_es_stream (ctrl, datastream, stats_handle,
-                             r_fpr, r_fprlen, options,
+                             r_fpr, r_fprlen,
+                             (opt.keyserver_options.import_options
+                              | IMPORT_NO_SECKEY),
                              keyserver_retrieval_screener, &screenerarg,
                              only_fprs? KEYORG_KS : 0,
                              source);
@@ -1549,12 +1766,11 @@ keyserver_get_chunk (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
    (DESC,NDESC).  Allowed search modes are keyid, fingerprint, and
    exact searches.  OVERRIDE_KEYSERVER gives an optional override
    keyserver. If (R_FPR,R_FPRLEN) are not NULL, they may return the
-   fingerprint of a single imported key.  If the FLAG bit
-   KEYSERVER_IMPORT_FLAG_QUICK is set, dirmngr is advised to use a
-   shorter timeout. */
+   fingerprint of a single imported key.  If QUICK is set, dirmngr is
+   advised to use a shorter timeout. */
 static gpg_error_t
 keyserver_get (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
-               struct keyserver_spec *override_keyserver, unsigned int flags,
+               struct keyserver_spec *override_keyserver, int quick,
                unsigned char **r_fpr, size_t *r_fprlen)
 {
   gpg_error_t err;
@@ -1567,7 +1783,7 @@ keyserver_get (ctrl_t ctrl, KEYDB_SEARCH_DESC *desc, int ndesc,
   for (;;)
     {
       err = keyserver_get_chunk (ctrl, desc, ndesc, &ndesc_used, stats_handle,
-                                 override_keyserver, flags, r_fpr, r_fprlen);
+                                 override_keyserver, quick, r_fpr, r_fprlen);
       if (!err)
         any_good = 1;
       if (err || ndesc_used >= ndesc)
@@ -1611,16 +1827,15 @@ keyserver_put (ctrl_t ctrl, strlist_t keyspecs)
 
       err = export_pubkey_buffer (ctrl, kspec->d,
                                   opt.keyserver_options.export_options,
-                                  NULL, 0, NULL,
+                                  NULL,
                                   &keyblock, &data, &datalen);
       if (err)
         log_error (_("skipped \"%s\": %s\n"), kspec->d, gpg_strerror (err));
       else
         {
-          if (!opt.quiet)
-            log_info (_("sending key %s to %s\n"),
-                      keystr (keyblock->pkt->pkt.public_key->keyid),
-                      ksurl?ksurl:"[?]");
+          log_info (_("sending key %s to %s\n"),
+                    keystr (keyblock->pkt->pkt.public_key->keyid),
+                    ksurl?ksurl:"[?]");
 
           err = gpg_dirmngr_ks_put (ctrl, data, datalen, keyblock);
           release_kbnode (keyblock);
@@ -1650,8 +1865,6 @@ keyserver_fetch (ctrl_t ctrl, strlist_t urilist, int origin)
   strlist_t sl;
   estream_t datastream;
   unsigned int save_options = opt.keyserver_options.import_options;
-  int any_success = 0;
-  gpg_error_t firsterr = 0;
 
   /* Switch on fast-import, since fetch can handle more than one
      import and we don't want each set to rebuild the trustdb.
@@ -1675,25 +1888,13 @@ keyserver_fetch (ctrl_t ctrl, strlist_t urilist, int origin)
 
           import_print_stats (stats_handle);
           import_release_stats_handle (stats_handle);
-          any_success = 1;
         }
       else
-        {
-          log_info (_("WARNING: unable to fetch URI %s: %s\n"),
-                    sl->d, gpg_strerror (err));
-          if (!firsterr)
-            firsterr = err;
-        }
+        log_info (_("WARNING: unable to fetch URI %s: %s\n"),
+                  sl->d, gpg_strerror (err));
       es_fclose (datastream);
     }
 
-  if (!urilist)
-    err = gpg_error (GPG_ERR_NO_NAME);
-  else if (any_success)
-    err = 0;
-  else
-    err = firsterr;
-
   opt.keyserver_options.import_options = save_options;
 
   /* If the original options didn't have fast import, and the trustdb
@@ -1701,7 +1902,7 @@ keyserver_fetch (ctrl_t ctrl, strlist_t urilist, int origin)
   if (!(opt.keyserver_options.import_options&IMPORT_FAST))
     check_or_update_trustdb (ctrl);
 
-  return err;
+  return 0;
 }
 
 
@@ -1749,7 +1950,7 @@ keyserver_import_cert (ctrl_t ctrl, const char *name, int dane_mode,
               xfree (filtstr);
               if (!err)
                 err = import_keys_es_stream (ctrl, key, NULL, fpr, fpr_len,
-                                             IMPORT_ONLY_PUBKEYS,
+                                             IMPORT_NO_SECKEY,
                                              NULL, NULL, KEYORG_DANE, NULL);
               restore_import_filter (save_filt);
             }
@@ -1758,7 +1959,7 @@ keyserver_import_cert (ctrl_t ctrl, const char *name, int dane_mode,
         {
           err = import_keys_es_stream (ctrl, key, NULL, fpr, fpr_len,
                                        (opt.keyserver_options.import_options
-                                        | IMPORT_ONLY_PUBKEYS),
+                                        | IMPORT_NO_SECKEY),
                                        NULL, NULL, 0, NULL);
         }
 
@@ -1806,10 +2007,43 @@ keyserver_import_cert (ctrl_t ctrl, const char *name, int dane_mode,
   return err;
 }
 
+/* Import key pointed to by a PKA record. Return the requested
+   fingerprint in fpr. */
+gpg_error_t
+keyserver_import_pka (ctrl_t ctrl, const char *name,
+                      unsigned char **fpr, size_t *fpr_len)
+{
+  gpg_error_t err;
+  char *url;
+
+  err = gpg_dirmngr_get_pka (ctrl, name, fpr, fpr_len, &url);
+  if (url && *url && fpr && fpr_len)
+    {
+      /* An URL is available.  Lookup the key. */
+      struct keyserver_spec *spec;
+      spec = parse_keyserver_uri (url, 1);
+      if (spec)
+	{
+	  err = keyserver_import_fprint (ctrl, *fpr, *fpr_len, spec, 0);
+	  free_keyserver_spec (spec);
+	}
+    }
+  xfree (url);
+
+  if (err)
+    {
+      xfree(*fpr);
+      *fpr = NULL;
+      *fpr_len = 0;
+    }
+
+  return err;
+}
+
 
 /* Import a key using the Web Key Directory protocol.  */
 gpg_error_t
-keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
+keyserver_import_wkd (ctrl_t ctrl, const char *name, int quick,
                       unsigned char **fpr, size_t *fpr_len)
 {
   gpg_error_t err;
@@ -1819,7 +2053,7 @@ keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
 
   /* We want to work on the mbox.  That is what dirmngr will do anyway
    * and we need the mbox for the import filter anyway.  */
-  mbox = mailbox_from_userid (name, 0);
+  mbox = mailbox_from_userid (name);
   if (!mbox)
     {
       err = gpg_error_from_syserror ();
@@ -1828,7 +2062,7 @@ keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
       return err;
     }
 
-  err = gpg_dirmngr_wkd_get (ctrl, mbox, flags, &key, &url);
+  err = gpg_dirmngr_wkd_get (ctrl, mbox, quick, &key, &url);
   if (err)
     ;
   else if (key)
@@ -1851,7 +2085,7 @@ keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
           xfree (filtstr);
           if (!err)
             err = import_keys_es_stream (ctrl, key, NULL, fpr, fpr_len,
-                                         IMPORT_ONLY_PUBKEYS,
+                                         IMPORT_NO_SECKEY,
                                          NULL, NULL, KEYORG_WKD, url);
 
         }
@@ -1867,3 +2101,85 @@ keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
   xfree (mbox);
   return err;
 }
+
+
+/* Import a key by name using LDAP */
+int
+keyserver_import_ldap (ctrl_t ctrl,
+                       const char *name, unsigned char **fpr, size_t *fprlen)
+{
+  (void)ctrl;
+  (void)name;
+  (void)fpr;
+  (void)fprlen;
+  return gpg_error (GPG_ERR_NOT_IMPLEMENTED); /*FIXME*/
+#if 0
+  char *domain;
+  struct keyserver_spec *keyserver;
+  strlist_t list=NULL;
+  int rc,hostlen=1;
+  struct srventry *srvlist=NULL;
+  int srvcount,i;
+  char srvname[MAXDNAME];
+
+  /* Parse out the domain */
+  domain=strrchr(name,'@');
+  if(!domain)
+    return GPG_ERR_GENERAL;
+
+  domain++;
+
+  keyserver=xmalloc_clear(sizeof(struct keyserver_spec));
+  keyserver->scheme=xstrdup("ldap");
+  keyserver->host=xmalloc(1);
+  keyserver->host[0]='\0';
+
+  snprintf(srvname,MAXDNAME,"_pgpkey-ldap._tcp.%s",domain);
+
+  FIXME("network related - move to dirmngr or drop the code");
+  srvcount=getsrv(srvname,&srvlist);
+
+  for(i=0;i<srvcount;i++)
+    {
+      hostlen+=strlen(srvlist[i].target)+1;
+      keyserver->host=xrealloc(keyserver->host,hostlen);
+
+      strcat(keyserver->host,srvlist[i].target);
+
+      if(srvlist[i].port!=389)
+	{
+	  char port[7];
+
+	  hostlen+=6; /* a colon, plus 5 digits (unsigned 16-bit value) */
+	  keyserver->host=xrealloc(keyserver->host,hostlen);
+
+	  snprintf(port,7,":%u",srvlist[i].port);
+	  strcat(keyserver->host,port);
+	}
+
+      strcat(keyserver->host," ");
+    }
+
+  free(srvlist);
+
+  /* If all else fails, do the PGP Universal trick of
+     ldap://keys.(domain) */
+
+  hostlen+=5+strlen(domain);
+  keyserver->host=xrealloc(keyserver->host,hostlen);
+  strcat(keyserver->host,"keys.");
+  strcat(keyserver->host,domain);
+
+  append_to_strlist(&list,name);
+
+  rc = gpg_error (GPG_ERR_NOT_IMPLEMENTED); /*FIXME*/
+       /* keyserver_work (ctrl, KS_GETNAME, list, NULL, */
+       /*                 0, fpr, fpr_len, keyserver); */
+
+  free_strlist(list);
+
+  free_keyserver_spec(keyserver);
+
+  return rc;
+#endif
+}
