@@ -23,7 +23,6 @@
 #include "../common/util.h"
 
 /* We keep all global options in the structure OPT.  */
-EXTERN_UNLESS_MAIN_MODULE
 struct
 {
   int verbose;		/* Verbosity level.  */
@@ -46,24 +45,21 @@ void gpgconf_failure (gpg_error_t err) GPGRT_ATTR_NORETURN;
 
 /* Component system.  Each component is a set of options that can be
  * configured at the same time.  If you change this, don't forget to
- * update gc_component[] in gpgconf-comp.c.  */
+ * update GC_COMPONENT in gpgconf-comp.c.  */
 typedef enum
   {
-    /* Any component, used as a wildcard arg.  */
-    GC_COMPONENT_ANY,
-
     /* The classic GPG for OpenPGP.  */
     GC_COMPONENT_GPG,
 
-    /* GPG for S/MIME.  */
-    GC_COMPONENT_GPGSM,
-
     /* The GPG Agent.  */
     GC_COMPONENT_GPG_AGENT,
 
     /* The Smardcard Daemon.  */
     GC_COMPONENT_SCDAEMON,
 
+    /* GPG for S/MIME.  */
+    GC_COMPONENT_GPGSM,
+
     /* The LDAP Directory Manager for CRLs.  */
     GC_COMPONENT_DIRMNGR,
 
@@ -72,7 +68,7 @@ typedef enum
 
     /* The number of components.  */
     GC_COMPONENT_NR
-  } gc_component_id_t;
+  } gc_component_t;
 
 
 /* Initialize the components.  */
